param
(
	[Parameter(Mandatory=$true)]
	[string]$vmName,

	[Parameter(Mandatory=$true)]
	[string]$imagePath,

	[Parameter(Mandatory=$true)]
	[int]$diskSizeGB,

	[Parameter(Mandatory=$true)]
	[string]$hardDiskDirectory,

	[Parameter(Mandatory=$true)]
	[int]$generation,

	[Parameter(Mandatory=$true)]
	[int]$cpuCount,

	[Parameter(Mandatory=$true)]
	[int]$ramMB,

	[Parameter(Mandatory=$true)]
	[int]$maxRamMB,

	[Parameter(Mandatory=$true)]
	[int]$maxIOPS,

	[Parameter(Mandatory=$true)]
	[bool]$vmqEnabled,

	[Parameter(Mandatory=$true)]
	[int]$networkVlan,

	[Parameter(Mandatory=$true)]
	[string]$networkSwitchName,

	[Parameter(Mandatory=$true)]
	[bool]$nestedVirtualizationEnabled,

	[Parameter(Mandatory=$true)]
	[bool]$macAddressSpoofingEnabled
)

$ErrorActionPreference = "Stop"

# Create worker directory

if(-not (Test-Path $hardDiskDirectory)) {
    New-Item -Path $hardDiskDirectory -ItemType Directory -Force | Out-Null
}

$vhdPath = Join-Path $hardDiskDirectory "$vmName.vhdx"

# Create differencing disk

try {
	New-VHD -Path $vhdPath -ParentPath $imagePath -Differencing | Out-Null
}
catch {
	Write-Host "Error creating VM disk: $($_.Exception.Message)"
	exit 1
}


# Create VM

try {
    $ramBytes = $ramMB * 1024L * 1024L
	New-VM -Name $vmName -MemoryStartupBytes $ramBytes -Generation $generation -VHDPath $vhdPath `
        -Path $hardDiskDirectory -SwitchName $networkSwitchName | Out-Null
}
catch {
	Write-Host "Error creating VM: $($_.Exception.Message)"
	exit 1
}


# Update CPU count

try {
	Set-VMProcessor -VMName $vmName -Count $cpuCount | Out-Null
}
catch {
	Write-Host "Error updating VM CPU count: $($_.Exception.Message)"
	exit 1
}


# Enable nested virtualization support

if ($nestedVirtualizationEnabled) {
    try {
	    Set-VMProcessor -VMName $vmName -ExposeVirtualizationExtensions $true | Out-Null
    }
    catch {
	    Write-Host "Error enabling CPU virtualization extensions: $($_.Exception.Message)"
	    exit 1
    }
}


# Disable secure boot

if ($generation -eq 2) {
    try {
	    Set-VMFirmware -VMName $vmName -EnableSecureBoot Off | Out-Null
    }
    catch {
	    Write-Host "Error disabling VM secure boot: $($_.Exception.Message)"
	    exit 1
    }
}


# Set dynamic memory

if ($maxRamMB -gt 0) {
    try {
        $ramBytes = $ramMB * 1024L * 1024L
        $maxRamBytes = $maxRamMB * 1024L * 1024L
	    Set-VMMemory -VMName $vmName -DynamicMemoryEnabled $true `
            -StartupBytes $ramBytes -MinimumBytes $ramBytes -MaximumBytes $maxRamBytes -Buffer 20 | Out-Null
    }
    catch {
	    Write-Host "Error setting VM dynamic memory: $($_.Exception.Message)"
	    exit 1
    }
}


# Update VLAN

if ($networkVlan -gt 0) {
    try {
	    Set-VMNetworkAdapterVlan -VMName $vmName -Access -VlanId $networkVlan | Out-Null
    }
    catch {
	    Write-Host "Error setting VM VLAN: $($_.Exception.Message)"
	    exit 1
    }
}


# Disable VMQ

if (-not $vmqEnabled) {
    try {
	    Set-VMNetworkAdapter -VMName $vmName -VmqWeight 0 | Out-Null
    }
    catch {
	    Write-Host "Error disabling VMQ: $($_.Exception.Message)"
	    exit 1
    }
}

# Enable MAC address spoofing

if ($macAddressSpoofingEnabled) {
    try {
	    Set-VMNetworkAdapter -VMName $vmName -MacAddressSpoofing On | Out-Null
    }
    catch {
	    Write-Host "Error enabling MAC address spoofing: $($_.Exception.Message)"
	    exit 1
    }
}



# Start VM

try {
	Start-VM -Name $vmName | Out-Null
}
catch {
	Write-Host "Error starting VM: $($_.Exception.Message)"
	exit 1
}